@@ -27,7 +27,7 @@ from tiddlyweb.store import (NoBagError, NoRecipeError, NoTiddlerError,
 from tiddlyweb.stores import StorageInterface
 from tiddlyweb.util import binary_tiddler
 
-__version__ = '3.0.0'
+__version__ = '3.0.1'
 
 #logging.basicConfig()
 #logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
@@ -62,6 +62,15 @@ first_revision_table = Table('first_revision', Base.metadata,
             index=True, nullable=False),
         UniqueConstraint('tiddler_id', 'first_id'))
 
+class sCurrentRevision(object):
+    pass
+
+class sFirstRevision(object):
+    pass
+
+mapper(sCurrentRevision, current_revision_table)
+mapper(sFirstRevision, first_revision_table)
+
 
 class sField(Base):
 
@@ -133,11 +142,11 @@ class sRevision(Base):
     fields=relationship('sField',
         cascade='all, delete-orphan',
         backref='fields',
-        lazy=False)
+        lazy=True)
     tags=relationship('sTag',
         cascade='all, delete-orphan',
         backref='tags',
-        lazy=False)
+        lazy=True)
     text=relationship('sText',
         cascade='all, delete-orphan',
         backref='revision_text',
@@ -171,14 +180,14 @@ class sTiddler(Base):
             cascade='delete, delete-orphan')
 
     current=relationship('sRevision',
-            lazy=False,
+            lazy=True,
             uselist=False,
             secondary=current_revision_table,
             primaryjoin=id==current_revision_table.c.tiddler_id,
             secondaryjoin=current_revision_table.c.current_id==sRevision.number)
 
     first=relationship('sRevision',
-            lazy=False,
+            lazy=True,
             uselist=False,
             secondary=first_revision_table,
             primaryjoin=id==first_revision_table.c.tiddler_id,
@@ -361,9 +370,11 @@ class Store(StorageInterface):
 
     def list_bag_tiddlers(self, bag):
         try:
-            query = self.session.query(sBag).filter(sBag.name == bag.name)
             try:
-                tiddlers = query.one().tiddlers
+                self.session.query(sBag.id).filter(
+                    sBag.name == bag.name).one()
+                tiddlers = self.session.query(sTiddler).filter(
+                        sTiddler.bag == bag.name).all()
             except NoResultFound, exc:
                 raise NoBagError('no results for bag %s, %s' % (bag.name, exc))
             self.session.close()
@@ -511,12 +522,12 @@ class Store(StorageInterface):
             if not tiddler.bag:
                 raise NoBagError('bag required to save')
             try:
-                sbag = self.session.query(sBag).filter(sBag.name
+                sbag = self.session.query(sBag.id).filter(sBag.name
                         == tiddler.bag).one()
             except NoResultFound, exc:
                 raise NoBagError('bag must exist for tiddler save: %s', exc)
-            stiddler = self._store_tiddler(tiddler)
-            tiddler.revision = stiddler.current.number
+            current_revision_number = self._store_tiddler(tiddler)
+            tiddler.revision = current_revision_number
             self.session.commit()
         except:
             self.session.rollback()
@@ -683,7 +694,7 @@ class Store(StorageInterface):
             srecipe = self.session.query(sRecipe).filter(
                     sRecipe.name == recipe.name).one()
         except NoResultFound:
-            srecipe = sRecipe(recipe.name, recipe.desc)
+            srecipe = sRecipe(recipe.name)
             self.session.add(srecipe)
         srecipe.desc = recipe.desc
         self._store_policy(srecipe, recipe.policy)
@@ -709,7 +720,7 @@ class Store(StorageInterface):
             tiddler.text = unicode(b64encode(tiddler.text))
 
         try:
-            stiddler = self.session.query(sTiddler).filter(
+            stiddler = self.session.query(sTiddler.id).filter(
                     and_(sTiddler.title==tiddler.title,
                         sTiddler.bag==tiddler.bag)).one()
             newTiddler = False
@@ -717,8 +728,7 @@ class Store(StorageInterface):
             stiddler = sTiddler(tiddler.title, tiddler.bag)
             self.session.add(stiddler)
             newTiddler = True
-
-        self.session.flush()
+            self.session.flush()
 
         srevision = sRevision()
         srevision.type = tiddler.type
@@ -745,11 +755,19 @@ class Store(StorageInterface):
                 self.session.add(sfield)
 
         self.session.flush()
-        stiddler.current = srevision
-        if newTiddler:
-            stiddler.first = srevision
 
-        return stiddler
+        current_revision = sCurrentRevision()
+        current_revision.tiddler_id = stiddler.id
+        current_revision.current_id = srevision.number
+        self.session.merge(current_revision)
+
+        if newTiddler:
+            first_revision = sFirstRevision()
+            first_revision.tiddler_id = stiddler.id
+            first_revision.first_id = srevision.number
+            self.session.merge(first_revision)
+
+        return srevision.number
 
     def _store_user(self, user):
         suser = sUser()
